@@ -1,7 +1,7 @@
 /*
  * libxlsxwriter
  *
- * Copyright 2014-2018, John McNamara, jmcnamara@cpan.org. See LICENSE.txt.
+ * Copyright 2014-2019, John McNamara, jmcnamara@cpan.org. See LICENSE.txt.
  */
 
 /**
@@ -72,14 +72,14 @@
  * The type for RGB colors in libxlsxwriter. The valid range is `0x000000`
  * (black) to `0xFFFFFF` (white). See @ref working_with_colors.
  */
-typedef int32_t lxw_color_t;
+typedef uint32_t lxw_color_t;
 
 #define LXW_FORMAT_FIELD_LEN            128
 #define LXW_DEFAULT_FONT_NAME           "Calibri"
 #define LXW_DEFAULT_FONT_FAMILY         2
 #define LXW_DEFAULT_FONT_THEME          1
 #define LXW_PROPERTY_UNSET              -1
-#define LXW_COLOR_UNSET                 -1
+#define LXW_COLOR_UNSET                 0x000000
 #define LXW_COLOR_MASK                  0xFFFFFF
 #define LXW_MIN_FONT_SIZE               1.0
 #define LXW_MAX_FONT_SIZE               409.0
@@ -480,15 +480,13 @@ extern "C" {
 #endif
 /* *INDENT-ON* */
 
-lxw_format *lxw_format_new();
+lxw_format *lxw_format_new(void);
 void lxw_format_free(lxw_format *format);
 int32_t lxw_format_get_xf_index(lxw_format *format);
 lxw_font *lxw_format_get_font_key(lxw_format *format);
 lxw_border *lxw_format_get_border_key(lxw_format *format);
 lxw_fill *lxw_format_get_fill_key(lxw_format *format);
 
-lxw_color_t lxw_format_check_color(lxw_color_t color);
-
 /**
  * @brief Set the font used in the cell.
  *
@@ -672,6 +670,9 @@ void format_set_font_script(lxw_format *format, uint8_t style);
  *
  * @image html format_set_num_format.png
  *
+ * To set a number format that matches an Excel format category such as "Date"
+ * or "Currency" see @ref ww_formats_categories.
+ *
  * The number system used for dates is described in @ref working_with_dates.
  *
  * For more information on number formats in Excel refer to the
@@ -749,6 +750,7 @@ void format_set_num_format(lxw_format *format, const char *num_format);
  *  - The dollar sign in the above format appears as the defined local currency
  *    symbol.
  *  - These formats can also be set via format_set_num_format().
+ *  - See also @ref ww_formats_categories.
  */
 void format_set_num_format_index(lxw_format *format, uint8_t index);
 
